import { EventEmitter } from '../../../stencil-public-runtime';
import { AdapterHost, MediaProviderAdapter } from '../../providers/MediaProvider';
import { Provider } from '../../providers/Provider';
import { Translation } from './lang/Translation';
import { MediaPlayer } from './MediaPlayer';
import { MediaType } from './MediaType';
import { Logger } from './PlayerLogger';
import { PlayerProps } from './PlayerProps';
import { ViewType } from './ViewType';
/**
 * The root component that encapsulates all providers, plugins and UI components. This is the
 * primary component you will interact with to set properties on the player, listen for events
 * and call methods.
 *
 * @slot - Used to pass in providers, plugins and UI components.
 */
export declare class Player implements MediaPlayer {
  provider?: AdapterHost;
  get adapter(): Promise<MediaProviderAdapter<any>> | undefined;
  private safeAdapterCall;
  private fullscreenController;
  private disposal;
  host: HTMLVmPlayerElement;
  container?: HTMLDivElement;
  onContainerChange(): void;
  /**
   * ------------------------------------------------------
   * Props
   * ------------------------------------------------------
   */
  /** @internal @readonly */
  logger: Logger;
  /** @inheritDoc */
  theme?: string;
  /** @inheritDoc */
  icons: string;
  /** @inheritDoc */
  paused: boolean;
  onPausedChange(): void;
  /** @inheritDoc @readonly */
  playing: boolean;
  /** @inheritDoc @readonly */
  duration: number;
  onDurationChange(): void;
  /** @inheritDoc @readonly */
  mediaTitle?: string;
  /** @inheritDoc @readonly */
  currentProvider?: Provider;
  /** @inheritDoc @readonly */
  currentSrc?: string;
  /** @inheritDoc @readonly */
  currentPoster?: string;
  /** @inheritDoc */
  currentTime: number;
  onCurrentTimeChange(): void;
  /** @inheritDoc */
  autoplay: boolean;
  /** @inheritDoc @readonly */
  ready: boolean;
  /** @inheritDoc @readonly */
  playbackReady: boolean;
  onPlaybackReadyChange(): void;
  /** @inheritDoc */
  loop: boolean;
  /** @inheritDoc */
  muted: boolean;
  onMutedChange(): void;
  /** @inheritDoc @readonly */
  buffered: number;
  /** @inheritDoc */
  playbackRate: number;
  private lastRateCheck;
  onPlaybackRateChange(newRate: number, prevRate: number): Promise<void>;
  /** @inheritDoc @readonly */
  playbackRates: number[];
  /** @inheritDoc */
  playbackQuality?: string;
  private lastQualityCheck?;
  onPlaybackQualityChange(newQuality: string, prevQuality: string): Promise<void>;
  /** @inheritDoc @readonly */
  playbackQualities: string[];
  /** @inheritDoc @readonly */
  seeking: boolean;
  /** @inheritDoc */
  debug: boolean;
  onDebugChange(): void;
  /** @inheritDoc @readonly */
  playbackStarted: boolean;
  /** @inheritDoc @readonly */
  playbackEnded: boolean;
  /** @inheritDoc @readonly */
  buffering: boolean;
  /** @inheritDoc */
  controls: boolean;
  /** @inheritDoc */
  isControlsActive: boolean;
  /** @inheritDoc @readonly */
  isSettingsActive: boolean;
  /** @inheritDoc */
  volume: number;
  onVolumeChange(): Promise<void>;
  /** @inheritDoc @readonly */
  isFullscreenActive: boolean;
  /** @inheritDoc */
  aspectRatio: string;
  /** @inheritDoc @readonly */
  viewType?: ViewType;
  onViewTypeChange(): void;
  /** @inheritDoc @readonly */
  isAudioView: boolean;
  /** @inheritDoc @readonly */
  isVideoView: boolean;
  /** @inheritDoc @readonly */
  mediaType?: MediaType;
  onMediaTypeChange(): void;
  /** @inheritDoc @readonly */
  isAudio: boolean;
  /** @inheritDoc @readonly */
  isVideo: boolean;
  /** @inheritDoc @readonly */
  isLive: boolean;
  /** @inheritDoc @readonly */
  isMobile: boolean;
  /** @inheritDoc @readonly */
  isTouch: boolean;
  /** @inheritDoc @readonly */
  isPiPActive: boolean;
  /** @inheritDoc @readonly */
  textTracks: never[];
  /** @inheritDoc @readonly */
  currentTextTrack: number;
  /** @inheritDoc @readonly */
  isTextTrackVisible: boolean;
  /** @inheritDoc */
  shouldRenderNativeTextTracks: boolean;
  /** @inheritDoc @readonly */
  audioTracks: never[];
  /** @inheritDoc @readonly */
  currentAudioTrack: number;
  /** @inheritDoc */
  autopause: boolean;
  /** @inheritDoc */
  playsinline: boolean;
  /** @inheritDoc */
  language: string;
  onLanguageChange(_: string, prevLanguage: string): void;
  /** @inheritDoc */
  translations: Record<string, Translation>;
  onTranslationsChange(): void;
  /** @inheritDoc @readonly */
  languages: string[];
  /** @inheritDoc @readonly */
  i18n: Translation;
  /**
   * ------------------------------------------------------
   * Events
   * ------------------------------------------------------
   */
  /** @inheritDoc */
  vmThemeChange: EventEmitter<PlayerProps['theme']>;
  /** @inheritDoc */
  vmPausedChange: EventEmitter<PlayerProps['paused']>;
  /** @inheritDoc */
  vmPlay: EventEmitter<void>;
  /** @inheritDoc */
  vmPlayingChange: EventEmitter<PlayerProps['playing']>;
  /** @inheritDoc */
  vmSeekingChange: EventEmitter<PlayerProps['seeking']>;
  /** @inheritDoc */
  vmSeeked: EventEmitter<void>;
  /** @inheritDoc */
  vmBufferingChange: EventEmitter<PlayerProps['buffering']>;
  /** @inheritDoc */
  vmDurationChange: EventEmitter<PlayerProps['duration']>;
  /** @inheritDoc */
  vmCurrentTimeChange: EventEmitter<PlayerProps['currentTime']>;
  /** @inheritDoc */
  vmReady: EventEmitter<void>;
  /** @inheritDoc */
  vmPlaybackReady: EventEmitter<void>;
  /** @inheritDoc */
  vmPlaybackStarted: EventEmitter<void>;
  /** @inheritDoc */
  vmPlaybackEnded: EventEmitter<void>;
  /** @inheritDoc */
  vmBufferedChange: EventEmitter<PlayerProps['buffered']>;
  /** @inheritDoc */
  vmError: EventEmitter<any>;
  onError(event: CustomEvent<any>): void;
  /** @inheritDoc */
  vmLoadStart: EventEmitter<void>;
  /** @inheritDoc */
  vmCurrentProviderChange: EventEmitter<PlayerProps['currentProvider']>;
  /** @inheritDoc */
  vmCurrentSrcChange: EventEmitter<PlayerProps['currentSrc']>;
  /** @inheritDoc */
  vmCurrentPosterChange: EventEmitter<PlayerProps['currentPoster']>;
  /** @inheritDoc */
  vmMediaTitleChange: EventEmitter<PlayerProps['mediaTitle']>;
  /** @inheritDoc */
  vmControlsChange: EventEmitter<PlayerProps['isControlsActive']>;
  /** @inheritDoc */
  vmPlaybackRateChange: EventEmitter<PlayerProps['playbackRate']>;
  /** @inheritDoc */
  vmPlaybackRatesChange: EventEmitter<PlayerProps['playbackRates']>;
  /** @inheritDoc */
  vmPlaybackQualityChange: EventEmitter<PlayerProps['playbackQuality']>;
  /** @inheritDoc */
  vmPlaybackQualitiesChange: EventEmitter<PlayerProps['playbackQualities']>;
  /** @inheritDoc */
  vmMutedChange: EventEmitter<PlayerProps['muted']>;
  /** @inheritDoc */
  vmVolumeChange: EventEmitter<PlayerProps['volume']>;
  /** @inheritDoc */
  vmViewTypeChange: EventEmitter<PlayerProps['viewType']>;
  /** @inheritDoc */
  vmMediaTypeChange: EventEmitter<PlayerProps['mediaType']>;
  /** @inheritDoc */
  vmLiveChange: EventEmitter<PlayerProps['isLive']>;
  /** @inheritDoc */
  vmTouchChange: EventEmitter<PlayerProps['isTouch']>;
  /** @inheritDoc */
  vmLanguageChange: EventEmitter<PlayerProps['language']>;
  /**
   * @inheritDoc
   */
  vmI18nChange: EventEmitter<PlayerProps['i18n']>;
  /**
   * @inheritDoc
   */
  vmTranslationsChange: EventEmitter<PlayerProps['translations']>;
  /** @inheritDoc */
  vmLanguagesChange: EventEmitter<PlayerProps['languages']>;
  /** @inheritDoc */
  vmFullscreenChange: EventEmitter<PlayerProps['isFullscreenActive']>;
  /** @inheritDoc */
  vmPiPChange: EventEmitter<PlayerProps['isPiPActive']>;
  /** @inheritDoc */
  vmTextTracksChange: EventEmitter<PlayerProps['textTracks']>;
  /** @inheritDoc */
  vmCurrentTextTrackChange: EventEmitter<PlayerProps['currentTextTrack']>;
  /** @inheritDoc */
  vmTextTrackVisibleChange: EventEmitter<PlayerProps['isTextTrackVisible']>;
  /** @inheritDoc */
  vmAudioTracksChange: EventEmitter<PlayerProps['audioTracks']>;
  /** @inheritDoc */
  vmCurrentAudioTrackChange: EventEmitter<PlayerProps['currentAudioTrack']>;
  /**
   * ------------------------------------------------------
   * Methods
   * ------------------------------------------------------
   */
  /** @inheritDoc */
  getProvider<InternalPlayerType = any>(): Promise<AdapterHost<InternalPlayerType> | undefined>;
  /** @internal */
  getAdapter<InternalPlayerType = any>(): Promise<MediaProviderAdapter<InternalPlayerType> | undefined>;
  /** @inheritDoc */
  play(): Promise<void | undefined>;
  /** @inheritDoc */
  pause(): Promise<void | undefined>;
  /** @inheritDoc */
  canPlay(type: string): Promise<boolean>;
  /** @inheritDoc */
  canAutoplay(): Promise<boolean>;
  /** @inheritDoc */
  canMutedAutoplay(): Promise<boolean>;
  /** @inheritDoc */
  canSetPlaybackRate(): Promise<boolean>;
  /** @inheritDoc */
  canSetPlaybackQuality(): Promise<boolean>;
  /** @inheritDoc */
  canSetFullscreen(): Promise<boolean>;
  /** @inheritDoc */
  enterFullscreen(options?: FullscreenOptions): Promise<void>;
  /** @inheritDoc */
  exitFullscreen(): Promise<void>;
  /** @inheritDoc */
  canSetPiP(): Promise<boolean>;
  /** @inheritDoc */
  enterPiP(): Promise<void | undefined>;
  /** @inheritDoc */
  exitPiP(): Promise<void | undefined>;
  /** @inheritDoc */
  canSetAudioTrack(): Promise<boolean>;
  /** @inheritDoc */
  setCurrentAudioTrack(trackId: number): Promise<void>;
  /** @inheritDoc */
  canSetTextTrack(): Promise<boolean>;
  /** @inheritDoc */
  setCurrentTextTrack(trackId: number): Promise<void>;
  /** @inheritDoc */
  canSetTextTrackVisibility(): Promise<boolean>;
  /** @inheritDoc */
  setTextTrackVisibility(isVisible: boolean): Promise<void>;
  /** @inheritDoc */
  extendLanguage(language: string, translation: Partial<Translation>): Promise<void>;
  constructor();
  connectedCallback(): void;
  componentWillLoad(): void;
  disconnectedCallback(): void;
  private rotateDevice;
  private getPlayerState;
  private calcAspectRatio;
  /**
   * Returns the inner container.
   */
  getContainer(): Promise<HTMLDivElement | undefined>;
  /** @internal Exposed for E2E testing. */
  callAdapter(method: keyof MediaProviderAdapter, value?: any): Promise<any>;
  private hasCustomControls;
  private genId;
  render(): any;
}
