import { EventEmitter } from '../../../stencil-public-runtime';
import { Logger } from '../../core/player/PlayerLogger';
import { ViewType } from '../../core/player/ViewType';
import { MediaProvider } from '../MediaProvider';
import { MediaCrossOriginOption, MediaFileProvider, MediaPreloadOption } from './MediaFileProvider';
import { MediaResource } from './MediaResource';
import { VideoFullscreenController } from './VideoFullscreenController';
import { VideoPresentationController } from './VideoPresentationController';
/**
 * Enables loading, playing and controlling media files via the
 * HTML5 [MediaElement](https://developer.mozilla.org/en-US/docs/Web/API/HTMLMediaElement) API.
 * This is used internally by the [`vime-audio`](audio.md) and [`vime-video`](video.md) components,
 * which are the preferred components to use for loading file-based media.
 *
 * > You don't interact with this component for passing player properties, controlling playback,
 * listening to player events and so on, that is all done through the `vime-player` component.
 *
 * @slot - Pass `<source>` and `<track>` elements to the underlying HTML5 media player.
 */
export declare class File implements MediaFileProvider<HTMLMediaElement>, MediaProvider<HTMLMediaElement> {
  private dispatch;
  private timeRAF?;
  private textTracksDisposal;
  private lazyLoader?;
  private wasPausedBeforeSeeking;
  private currentSrcSet;
  private prevMediaEl?;
  private mediaEl?;
  private mediaQueryDisposal;
  host: HTMLVmFileElement;
  vmPoster?: HTMLVmPosterElement;
  /** @internal Whether an external SDK will attach itself to the media player and control it. */
  willAttach: boolean;
  /** @inheritdoc */
  crossOrigin?: MediaCrossOriginOption;
  /** @inheritdoc */
  preload?: MediaPreloadOption;
  /** @inheritdoc */
  poster?: string;
  /**
   * The title of the current media.
   */
  mediaTitle?: string;
  onMediaTitleChange(): void;
  onPosterChange(): void;
  /** @inheritdoc */
  controlsList?: string;
  /** @inheritdoc */
  autoPiP?: boolean;
  /** @inheritdoc */
  disablePiP?: boolean;
  /** @inheritdoc */
  disableRemotePlayback?: boolean;
  /**
   * Whether to use an `audio` or `video` element to play the media.
   */
  viewType?: ViewType;
  onViewTypeChange(): void;
  /**
   * The playback rates that are available for this media.
   */
  playbackRates: number[];
  /** @internal */
  language: string;
  /** @internal */
  autoplay: boolean;
  /** @internal */
  controls: boolean;
  /** @internal */
  logger?: Logger;
  /** @internal */
  loop: boolean;
  /** @internal */
  muted: boolean;
  /** @internal */
  playsinline: boolean;
  /** @internal */
  noConnect: boolean;
  /** @internal */
  paused: boolean;
  /** @internal */
  currentTime: number;
  /** @internal */
  volume: number;
  /** @internal */
  playbackReady: boolean;
  /** @internal */
  playbackStarted: boolean;
  /** @internal */
  vmLoadStart: EventEmitter<void>;
  /**
   * Emitted when an error has occurred.
   */
  vmError: EventEmitter<any>;
  /**
   * Emitted when the underlying media element changes.
   */
  vmMediaElChange: EventEmitter<HTMLAudioElement | HTMLVideoElement | undefined>;
  /**
   * Emitted when the child `<source />` elements are modified.
   */
  vmSrcSetChange: EventEmitter<MediaResource[]>;
  constructor();
  connectedCallback(): void;
  componentDidRender(): void;
  componentDidLoad(): void;
  disconnectedCallback(): void;
  private initLazyLoader;
  private refresh;
  private didSrcSetChange;
  private onSrcSetChange;
  private hasCustomPoster;
  private cancelTimeUpdates;
  private requestTimeUpdates;
  private getMediaType;
  private onLoadedMetadata;
  private onProgress;
  private onPlay;
  private onPause;
  private onPlaying;
  private onSeeking;
  private onSeeked;
  private onRateChange;
  private onVolumeChange;
  private onDurationChange;
  private onWaiting;
  private onSuspend;
  private onEnded;
  private onError;
  private attemptToPlay;
  private togglePiPInChrome;
  private togglePiPInSafari;
  private togglePiP;
  private onEnterPiP;
  private onLeavePiP;
  protected presentationController: VideoPresentationController;
  protected addPresentationControllerListeners(): void;
  protected fullscreenController: VideoFullscreenController;
  /** @internal */
  getAdapter(): Promise<{
    getInternalPlayer: () => Promise<HTMLMediaElement>;
    play: () => Promise<void | undefined>;
    pause: () => Promise<void | undefined>;
    canPlay: (type: any) => Promise<boolean>;
    setCurrentTime: (time: number) => Promise<void>;
    setMuted: (muted: boolean) => Promise<void>;
    setVolume: (volume: number) => Promise<void>;
    canSetPlaybackRate: () => Promise<boolean>;
    setPlaybackRate: (rate: number) => Promise<void>;
    canSetPiP: () => Promise<boolean>;
    enterPiP: () => Promise<any>;
    exitPiP: () => Promise<any>;
    canSetFullscreen: () => Promise<boolean>;
    enterFullscreen: () => Promise<void>;
    exitFullscreen: () => Promise<void>;
    setCurrentTextTrack: (trackId: number) => Promise<void>;
    setTextTrackVisibility: (isVisible: boolean) => Promise<void>;
  }>;
  /** @internal */
  currentTextTrack: number;
  /** @internal */
  hasCustomTextManager: boolean;
  onHasCustomTextManagerChange(): void;
  /** @internal */
  isTextTrackVisible: boolean;
  /** @internal */
  shouldRenderNativeTextTracks: boolean;
  onShouldRenderNativeTextTracksChange(): void;
  onProviderConnect(event: Event): void;
  onProviderDisconnect(event: Event): void;
  private getFilteredTextTracks;
  private listenToTextTracksForChanges;
  private onTextTracksChange;
  private toggleTextTrackModes;
  render(): any;
}
